/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "CvfdStructuringElement.h"

using namespace std;
using namespace FD;

class CvfdErode;
   
DECLARE_NODE(CvfdErode)
/*Node
 *
 * @name CvfdErode
 * @category FDOPENCV_ORG:Morphological Operations
 * @description Erodes image by using arbitrary structuring element
 *
 * @input_name IMAGEIN
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @input_name ELEMENT
 * @input_description Input element
 * @input_type CvfdStructuringElement
 *
 * @output_name IMAGEOUT
 * @output_description Output image
 * @output_type CvfdImage
 *
 * @parameter_name ITERATIONS
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Number of times dilation is applied
 *
 END*/
   
   
class CvfdErode : public BufferedNode {
  //Input ID
  int m_imageInID;
  int m_elementID;
  //Output ID
  int m_imageOutID;
  //Parameters
  int m_iterations;
      
public:
  CvfdErode(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_imageInID = addInput("IMAGEIN");
    m_elementID = addInput("ELEMENT");
    //add outputs
    m_imageOutID = addOutput("IMAGEOUT");
    //Initialize parameters         
    m_iterations = dereference_cast<int>(parameters.get("ITERATIONS"));
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    //Read the inputs
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   
    RCPtr<CvfdStructuringElement> elementPtr = getInput(m_elementID,count);
         
    //Handle
    CvfdImage* image= new CvfdImage(&(*imagePtr));  
         
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    __BEGIN__;
    OPENCV_CALL(cvErode( imagePtr->getImage(), image->getImage(), elementPtr->getStructuringElement(), m_iterations));
    __END__;
    cvSetErrMode( status );
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to erode the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      }
         
    out[count] = ObjectRef(image);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdErode)
};
