/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdContours.h"
#include "CvfdImage.h"

using namespace std;
using namespace FD;

class CvfdFindContours;
   
DECLARE_NODE(CvfdFindContours)
/*Node
 *
 * @name CvfdFindContours
 * @category FDOPENCV_ORG:Contours
 * @description Finds contours in binary image
 *
 * @input_name IMAGE
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @parameter_name MODE
 * @parameter_type string
 * @parameter_value CV_RETR_EXTERNAL;CV_RETR_LIST;CV_RETR_CCOMP;CV_RETR_TREE
 * @parameter_description Retrieval mode
 * CV_RETR_EXTERNAL: retrive only the extreme outer contours
 * CV_RETR_LIST: retrieve all the contours and puts them in the list
 * CV_RETR_CCOMP: retrieve all the contours and organizes them into two-level hierarchy: top level are external boundaries of the components, second level are bounda boundaries of the holes
 * CV_RETR_TREE: retrieve all the contours and reconstructs the full hierarchy of nested contours
 *
 * @parameter_name METHOD
 * @parameter_type string
 * @parameter_value CV_CHAIN_APPROX_NONE;CV_CHAIN_APPROX_SIMPLE;CV_CHAIN_APPROX_TC89_L1;CV_LINK_RUNS
 * @parameter_description Approximation method
 * CV_CHAIN_CODE: output contours in the Freeman chain code. All other methods output polygons (sequences of vertices).
 * CV_CHAIN_APPROX_NONE: translate all the points from the chain code into points;
 * CV_CHAIN_APPROX_SIMPLE: compress horizontal, vertical, and diagonal segments, that is, the function leaves only their ending points;
 * CV_CHAIN_APPROX_TC89_L1,CV_CHAIN_APPROX_TC89_KCOS: apply one of the flavors of Teh-Chin chain approximation algorithm.
 * CV_LINK_RUNS: use completely different contour retrieval algorithm via linking of horizontal segments of 1?s. Only CV_RETR_LIST retrieval mode can be used with this method.
 *
 * @parameter_name AREA_MIN
 * @parameter_type float
 * @parameter_value 0
 * @parameter_description All the contours which are superior to this value will be retrived.
 *
 * @output_name CONTOURS
 * @output_description Contain the pointer to the first outer contour
 * @output_type CvfdContours
 *
 * @output_name NBCONTOURS
 * @output_description Number of contours
 * @output_type int
 *
 END*/
   
   
class CvfdFindContours : public BufferedNode {
  //Input ID
  int m_imageID;
  //Outputs ID
  int m_contoursID;
  int m_nbContoursID;
  //Parameters
  string m_mode;
  string m_method;
  float m_areaMin;
      
  //map of the mode and methode
  map<string,int> m_modeMap;
  map<string,int> m_methodMap;
      
  //Memory Storage for the contours
  CvMemStorage* m_storage;
      
public:
  CvfdFindContours(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_imageID = addInput("IMAGE");
    //add outputs
    m_contoursID = addOutput("CONTOURS");
    m_nbContoursID = addOutput("NBCONTOURS");
    //Initialize parameters         
    m_mode = object_cast<String>(parameters.get("MODE"));
    m_method = object_cast<String>(parameters.get("METHOD"));
    m_areaMin = object_cast<Float>(parameters.get("AREA_MIN"));
         
    //Initialize the mode map
    m_modeMap["CV_RETR_EXTERNAL"] = CV_RETR_EXTERNAL;
    m_modeMap["CV_RETR_LIST"] = CV_RETR_LIST;
    m_modeMap["CV_RETR_CCOMP"] = CV_RETR_CCOMP;
    m_modeMap["CV_RETR_TREE"] = CV_RETR_TREE;
    //Initialize the method map
    m_methodMap["CV_CHAIN_CODE"] = CV_CHAIN_CODE;
    m_methodMap["CV_CHAIN_APPROX_NONE"] = CV_CHAIN_APPROX_NONE;
    m_methodMap["CV_CHAIN_APPROX_SIMPLE"] = CV_CHAIN_APPROX_SIMPLE;
    m_methodMap["CV_CHAIN_APPROX_TC89_L1"] = CV_CHAIN_APPROX_TC89_L1;
    m_methodMap["CV_LINK_RUNS"] = CV_LINK_RUNS;
    //Initialze the memory storage
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent ); 
    __BEGIN__;
    OPENCV_CALL(m_storage = cvCreateMemStorage(0));
    __END__;
    cvSetErrMode( status );       
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to find contours: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      }         
  }
      
  ~CvfdFindContours()
  {
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent ); 
    __BEGIN__;
    OPENCV_CALL(cvReleaseMemStorage(&m_storage));
    __END__;
    cvSetErrMode( status );         
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to find contours: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      }         
  }
      
      
  void calculate(int output_id, int count, Buffer &out)
  {   
    //Read the inputs
    RCPtr<CvfdImage> imagePtr = getInput(m_imageID,count);
    //Handle
    CvfdImage* image = new CvfdImage(imagePtr->getImage());
    CvSeq* contours; 
    int nbContours=0;
    double area;
    CvContourScanner scanner;
         
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent ); 
    __BEGIN__;
    OPENCV_CALL(cvClearMemStorage( m_storage ));  
    OPENCV_CALL(scanner = cvStartFindContours( image->getImage(), m_storage
					       , sizeof(CvContour)
					       , m_modeMap[m_mode]
					       , m_methodMap[m_method]
					       , cvPoint(0,0) ));
    OPENCV_CALL(contours = cvFindNextContour( scanner ));
    while(contours!=NULL)
      {
	OPENCV_CALL(area = fabs(cvContourArea( contours )));
	if( area > m_areaMin)
	  {
	    OPENCV_CALL(cvSubstituteContour( scanner, contours ));
	    nbContours++;
	  }
	else
	  {
	    OPENCV_CALL(cvSubstituteContour( scanner, NULL));
	  }
	OPENCV_CALL(contours = cvFindNextContour( scanner ));
      }
    contours = cvEndFindContours( &scanner );

    __END__;
    cvSetErrMode( status );
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to find contours: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      }         
    delete image;
    CvfdContours* Contours = new CvfdContours();
    Contours->setNbContours(nbContours);
    Contours->setContours(contours);
    Contours->setFirstContours(contours);
    (*(outputs[m_contoursID].buffer))[count] = ObjectRef(Contours); 
    (*(outputs[m_nbContoursID].buffer))[count] = ObjectRef(Int::alloc(nbContours));
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdFindContours)
};
