/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "CvfdColor.h"
#include <math.h>

using namespace std;
using namespace FD;

class CvfdLine;
   
DECLARE_NODE(CvfdLine)
/*Node
 *
 * @name CvfdLine
 * @category FDOPENCV_ORG:Draw
 * @description Draws contour outlines or interiors in the image
 *
 * @input_name IMAGEIN
 * @input_description the input image
 * @input_type CvfdImage
 *
 * @input_name COLOR
 * @input_description the BGR color of the rectangle
 * @input_type CvfdColor
 *
 * @input_name X1
 * @input_description x coordinate to the first extremity of the line
 * @input_type int
 *
 * @input_name Y1
 * @input_description y coordinate to the first extremity of the line
 * @input_type int
 *
 * @input_name X2
 * @input_description x coordinate to the second extremity of the line
 * @input_type int
 *
 * @input_name Y2
 * @input_description y coordinate to the second extremity of the line
 * @input_type int
 *
 * @parameter_name THICKNESS
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Thickness of lines that make up the rectangle. Negative values, make the function to draw a filled rectangle
 *
 * @parameter_name LINE_TYPE
 * @parameter_type string
 * @parameter_value 8;4;CV_AA
 * @parameter_description Type of the contour segments
 *
 * @parameter_name SHIFT
 * @parameter_type int
 * @parameter_value 0
 * @parameter_description Number of fractional bits in the point coordinates
 *
 * @output_name IMAGEOUT
 * @output_description The result image
 * @output_type CvfdImage
 *
 END*/
inline std::string CINT(int value)
{
  std::stringstream sstr;
  sstr << value;
  return(sstr.str());
}   
   
class CvfdLine : public BufferedNode {
      
  //Input ID
  int m_colorID;
  int m_imageInID;
  int m_x1ID;
  int m_y1ID;
  int m_x2ID;
  int m_y2ID; 
  //Output ID
  int m_imageOutID;
  //Parameter
  string m_lineType;
  int m_thickness;
  int m_shift; 
  //map of the line_type
  map<string,int> m_lineTypeMap;
      
public:
  CvfdLine(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_colorID = addInput("COLOR");
    m_imageInID = addInput("IMAGEIN");
    m_x1ID = addInput("X1");
    m_y1ID = addInput("Y1");
    m_x2ID = addInput("X2");
    m_y2ID = addInput("Y2");         
    //add outputs
    m_imageOutID = addOutput("IMAGEOUT");
         
    //Initialize parameters         
    m_thickness = dereference_cast<int>(parameters.get("THICKNESS"));
    m_shift = dereference_cast<int>(parameters.get("SHIFT"));         
    m_lineType = object_cast<String>(parameters.get("LINE_TYPE"));         
         
    //Initialize lineType map
    m_lineTypeMap["8"] = 8;
    m_lineTypeMap["4"] = 4;
    m_lineTypeMap["CV_AA"] = CV_AA;         
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {           
    //Read the inputs
    RCPtr<CvfdColor> colorPtr = getInput(m_colorID,count);
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);
    RCPtr<Int> x1Ptr = getInput(m_x1ID,count);
    RCPtr<Int> y1Ptr = getInput(m_y1ID,count);
    RCPtr<Float> x2Ptr = getInput(m_x2ID,count);
    RCPtr<Float> y2Ptr = getInput(m_y2ID,count);         
    //Handle
    CvfdImage* image = new CvfdImage(&(*imagePtr));
         
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );         
    __BEGIN__; 
    OPENCV_CALL( cvLine( image->getImage(), cvPoint(*x1Ptr,*y1Ptr)
			 , cvPoint(*x2Ptr,*y2Ptr), CV_RGB( (colorPtr->getColor()).val[0], (colorPtr->getColor()).val[1], (colorPtr->getColor()).val[2] )
			 , m_thickness ,m_lineTypeMap[m_lineType], m_shift ));         
    __END__;
    cvSetErrMode( status );
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to draw a line: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      } 
         
    out[count] = ObjectRef(image);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdLine)
};
