/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include <math.h>

using namespace std;
using namespace FD;

class CvfdOpticalFlowLK;
   
DECLARE_NODE(CvfdOpticalFlowLK)
/*Node
 *
 * @name CvfdOpticalFlowLK
 * @category FDOPENCV_ORG:Video
 * @description Calculates optical flow for two images
 *
 * @input_name PREV
 * @input_description First image
 * @input_type CvfdImage
 *
 * @input_name CURR
 * @input_description Second image
 * @input_type CvfdImage
 *
 * @parameter_name WIN_SIZE_X
 * @parameter_type int
 * @parameter_value 3
 * @parameter_description Size of the averaging window used for grouping pixels (width)
 *
 * @parameter_name WIN_SIZE_Y
 * @parameter_type int
 * @parameter_value 3
 * @parameter_description Size of the averaging window used for grouping pixels (height)
 *
 * @output_name VEL_X
 * @output_description Horizontal component of the optical flow
 * @output_type CvfdImage
 *
 * @output_name VEL_Y
 * @output_description Vertical component of the optical flow
 * @output_type CvfdImage
 *
 END*/
   
class CvfdOpticalFlowLK : public BufferedNode {
      
  //Input ID
  int m_prevID;
  int m_currID;
  //Output ID
  int m_velxID;
  int m_velyID;
  //Parameters
  int m_winSizeX;
  int m_winSizeY;
      
public:
  CvfdOpticalFlowLK(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_prevID = addInput("PREV");
    m_currID = addInput("CURR");
         
    //add outputs
    m_velxID = addOutput("VEL_X");
    m_velyID = addOutput("VEL_Y");

    //Initialize parameters         
    m_winSizeX = dereference_cast<int>(parameters.get("WIN_SIZE_X"));
    m_winSizeY = dereference_cast<int>(parameters.get("WIN_SIZE_Y"));
         
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {           
    //Read the inputs
    RCPtr<CvfdImage> prevPtr = getInput(m_prevID,count);
    RCPtr<CvfdImage> currPtr = getInput(m_currID,count);
         
    //Handle
         
    CvfdImage* prev = prevPtr->gray();
    CvfdImage* curr = currPtr->gray();
    CvfdImage* velx;
    CvfdImage* vely;
         
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    __BEGIN__; 
    OPENCV_CALL( velx = new CvfdImage( cvGetSize(prevPtr->getImage()), IPL_DEPTH_32F, 1) );
    OPENCV_CALL( vely = new CvfdImage( cvGetSize(prevPtr->getImage()), IPL_DEPTH_32F, 1) );
    OPENCV_CALL( cvCalcOpticalFlowLK( prev->getImage(), curr->getImage()
				      , cvSize(m_winSizeX,m_winSizeY)
				      , velx->getImage(), vely->getImage() )); 
    __END__;
    cvSetErrMode( status );
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to calculate the optical flow: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      } 
    delete prev;
    delete curr;
    (*(outputs[m_velxID].buffer))[count] = ObjectRef(velx); 
    (*(outputs[m_velyID].buffer))[count] = ObjectRef(vely); 
  }
  NO_ORDER_NODE_SPEEDUP(CvfdOpticalFlowLK);
};
